"""Диаграмма состояний.

Описание - https://mermaid-js.github.io/mermaid/#/stateDiagram

Не поддерживается:
- concurrency
- direction
"""

import logging
from enum import Enum

from konstantin_docs.dia.base import BaseDiagram
from konstantin_docs.dia.base import Image
from konstantin_docs.service import kroki

from typing_extensions import Self

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())


TEMPL_DIA: str = """stateDiagram-v2
{states}
{trans}"""
TEMPL_STATE1: str = "{label}"
TEMPL_STATE2: str = """state {label} {{
{states}
{trans}
}}"""  # noqa: JS101
TEMPL_STATE3: str = "state {label} {modif}"
TEMPL_TRANS: str = "{begin} --> {end} {desc}"
TEMPL_NOTE: str = """note {pos} of {state}
{text}
end note"""


class Note:
    """Пояснение к состоянию."""

    class PosEnum(Enum):
        """Положение заметки."""

        left = "left"
        right = "right"

    __text: str
    __pos: PosEnum
    __state_label: str

    def __init__(self: Self, text: str, pos: PosEnum = PosEnum.left) -> None:
        """Пояснение к состоянию.

        :param text: текст пояснения
        :param pos: положение пояснения, слева или справа
        """
        self.__text = text
        self.__pos = pos

    def set_state_label(self: Self, label: str) -> None:
        """Установить название состояния.

        :param label: название состояния
        """
        self.__state_label = label

    def __repr__(self: Self) -> str:
        """Represent as string.

        :return: string representation
        """
        return TEMPL_NOTE.format(
            pos=self.__pos.value,
            state=self.__state_label,
            text=self.__text,
        )


class State:
    """Состояние."""

    class ModifEnum(Enum):
        """Разновидности состояний."""

        standard = ""
        choice = "<<choice>>"
        fork = "<<fork>>"
        join = "<<join>>"

    __label: str
    __states: list[Self] | None
    __trans: list["Trans"] | None
    __modif: ModifEnum
    __note: Note | None

    def __init__(
        self: Self,
        label: str,
        states: list[Self] | None = None,
        trans: list["Trans"] | None = None,
        modif: ModifEnum = ModifEnum.standard,
        note: Note | None = None,
    ) -> None:
        """Состояние.

        :param label: название состояния
        :param states: список состояний
        :param trans: список переходов
        :param modif: модификатор ModifEnum
        :param note: пояснение
        """
        self.__label = label
        self.__states = states
        self.__trans = trans
        self.__modif = modif
        self.__note = note
        if self.__note is not None:
            self.__note.set_state_label(self.__label)

    @property
    def label(self: Self) -> str:
        """Label.

        :return: label
        """
        return self.__label

    def __repr__(self: Self) -> str:
        """Represent as string.

        :return: string representation
        """
        out: str = ""
        if self.__states is not None and self.__trans is not None:
            out += TEMPL_STATE2.format(
                label=self.__label,
                states="\n".join([repr(s) for s in self.__states]),
                trans="\n".join([repr(t) for t in self.__trans]),
            )
        elif self.__modif == self.ModifEnum.standard:
            out += TEMPL_STATE1.format(
                label=self.__label,
            )
        else:
            out += TEMPL_STATE3.format(
                label=self.__label,
                modif=self.__modif.value,
            )
        if self.__note is not None:
            out += "\n" + repr(self.__note)
        return out


class Trans:
    """Переход."""

    __begin: State | None
    __end: State | None
    __desc: str

    def __init__(
        self: Self,
        begin: State | None,
        end: State | None,
        desc: str = "",
    ) -> None:
        """Переход.

        :param begin: начальное состояние
        :param end: конечное состояние
        :param desc: описание
        """
        self.__begin = begin
        self.__end = end
        self.__desc = desc

    def __repr__(self: Self) -> str:
        """Reperesent as string.

        :return: string representation
        """
        return TEMPL_TRANS.format(
            begin=self.__begin.label if self.__begin is not None else "[*]",
            end=self.__end.label if self.__end is not None else "[*]",
            desc=": " + self.__desc if self.__desc else "",
        )


class Diagram(BaseDiagram):
    """Класс диаграммы."""

    __states: list[State]
    __trans: list[Trans]

    def __init__(
        self: Self,
        filename: str,
        states: list[State],
        trans: list[Trans],
    ) -> None:
        """Класс диаграммы.

        :param filename: имя файла
        :param states: список состояний
        :param trans: список переходов
        """
        super().__init__(filename)
        self.__states = states
        self.__trans = trans

    def get_images(self: Self) -> tuple[Image]:
        """Возвращает кортеж изображений.

        :return: кортеж изображений
        """
        images: list[Image] = []
        text = repr(self)
        images.append(self._get_text_file(".puml"))
        try:
            for fmt in (kroki.OutputFormats.SVG,):
                images.append(
                    Image(
                        filename=self.filename + "." + fmt.value,
                        content=kroki.get_image(
                            source=text,
                            diagram_type=kroki.DiagramTypes.MERMAID,
                            output_format=fmt,
                        ),
                    ),
                )
        except RuntimeError as exc:
            logger.exception(exc)
        return tuple(images)

    def __repr__(self: Self) -> str:
        """Represent as string.

        :return: string representation
        """
        return TEMPL_DIA.format(
            states="\n".join([repr(s) for s in self.__states]),
            trans="\n".join([repr(t) for t in self.__trans]),
        )



"""tupadr3 lib."""

from konstantin_docs.dia.c4.base import BaseSprite as _BaseSprite
from konstantin_docs.dia.c4.base import BaseSprites as _BaseSprites

from .tupadr3_lib.devicons import DeviconsLib
from .tupadr3_lib.devicons2 import Devicons2Lib
from .tupadr3_lib.font_awesome_5 import FontAwesome5Lib


class _Sprite(_BaseSprite):
    """Tupadr3 library.

    https://github.com/tupadr3/plantuml-icon-font-sprites
    """

    def __init__(
        self: "_Sprite",
        common_sprite: str,
        sprite: _BaseSprites,
    ) -> None:
        """Create sprite tupadr3."""
        super().__init__("tupadr3/common", common_sprite, sprite)


class Devicons(_Sprite):
    """Font Awedome 5."""

    def __init__(self: "Devicons", sprite: DeviconsLib) -> None:
        """Create sprite tupadr3."""
        super().__init__("tupadr3/devicons", sprite)


class Devicons2(_Sprite):
    """Font Awedome 5."""

    def __init__(self: "Devicons2", sprite: Devicons2Lib) -> None:
        """Create sprite tupadr3."""
        super().__init__("tupadr3/devicons2", sprite)


class FontAwesome5(_Sprite):
    """Font Awedome 5."""

    def __init__(self: "_Sprite", sprite: FontAwesome5Lib) -> None:
        """Create sprite tupadr3."""
        super().__init__("tupadr3/font-awesome-5", sprite)



"""Отношения."""

from .base import BaseC4Element as _BaseC4Element
from .base import BaseRelation as _BaseRelation
from .tag import RelTag as _RelTag


class Rel(_BaseRelation):
    """Relation."""

    def __init__(
        self: "Rel",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает Relation."""
        super().__init__(
            kind="Rel",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelBack(_BaseRelation):
    """RelBack."""

    def __init__(
        self: "RelBack",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает Relation."""
        super().__init__(
            kind="Rel_Back",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelNeighbor(_BaseRelation):
    """Rel_Neighbor."""

    def __init__(
        self: "RelNeighbor",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает Relation."""
        super().__init__(
            kind="Rel_Neighbor",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelR(_BaseRelation):
    """RelR."""

    def __init__(
        self: "RelR",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает RelR."""
        super().__init__(
            kind="Rel_R",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelL(_BaseRelation):
    """RelL."""

    def __init__(
        self: "RelL",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает RelR."""
        super().__init__(
            kind="Rel_L",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelU(_BaseRelation):
    """RelU."""

    def __init__(
        self: "RelU",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает RelU."""
        super().__init__(
            kind="Rel_U",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class RelD(_BaseRelation):
    """RelD."""

    def __init__(
        self: "RelD",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает RelD."""
        super().__init__(
            kind="Rel_D",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class BiRel(_BaseRelation):
    """RelD."""

    def __init__(
        self: "BiRel",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает BiRel."""
        super().__init__(
            kind="BiRel",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class BiRelR(_BaseRelation):
    """BiRelR."""

    def __init__(
        self: "BiRelR",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает BiRelR."""
        super().__init__(
            kind="BiRel_R",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class BiRelL(_BaseRelation):
    """BiRelL."""

    def __init__(
        self: "BiRelL",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает BiRelL."""
        super().__init__(
            kind="BiRel_L",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class BiRelU(_BaseRelation):
    """BiRelU."""

    def __init__(
        self: "BiRelU",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает BiRelU."""
        super().__init__(
            kind="BiRel_U",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )


class BiRelD(_BaseRelation):
    """BiRelD."""

    def __init__(
        self: "BiRelD",
        links: tuple[_BaseC4Element, _BaseC4Element],
        label: str,
        techn: str = "",
        descr: str = "",
        link: str | None = None,
        tags: tuple[_RelTag, ...] | None = None,
    ) -> None:
        """Создает BiRelD."""
        super().__init__(
            kind="BiRel_D",
            links=links,
            label=label,
            techn=techn,
            descr=descr,
            link=link,
            tags=tags,
        )



"""Поддержка тегов."""

from .base import BaseSprite as _BaseSprite
from .base import BaseTag as _BaseTag


class ElementTag(_BaseTag):
    """Тег для элементов."""

    __bg_color: str | None
    __font_color: str | None
    __border_color: str | None

    def __init__(
        self: "ElementTag",
        tag_stereo: str,
        bg_color: str | None = None,
        font_color: str | None = None,
        border_color: str | None = None,
    ) -> None:
        """Создает тег для элементов."""
        super().__init__(tag_stereo)
        self.__bg_color = bg_color
        self.__font_color = font_color
        self.__border_color = border_color

    def repr_declaration(self: "ElementTag") -> str:
        """Объявление тега в начале файла."""
        return (
            f'AddElementTag("{self.repr_tag()}"'
            f"{self._repr_font_color()}"
            f"{self._repr_border_color()}"
            ")"
        )

    def _repr_border_color(self: "ElementTag") -> str:
        if self.__border_color is None:
            return ""
        return f', $borderColor="{self.__border_color}"'

    def _repr_font_color(self: "ElementTag") -> str:
        if self.__font_color is None:
            return ""
        return f', $fontColor="{self.__font_color}"'


class RelTag(_BaseTag):
    """Тег для отношений."""

    def __init__(
        self: "RelTag",
        tag_stereo: str,
        text_color: str | None = None,
        line_color: str | None = None,
        line_style: str | None = None,
        sprite: _BaseSprite | None = None,
        techn: str | None = None,
        legent_text: str | None = None,
        legend_sprite: str | None = None,
    ) -> None:
        """Создает тег для отношений."""
        super().__init__(tag_stereo)
        self.__text_color = text_color
        self.__line_color = line_color
        self.__line_style = line_style

    def repr_declaration(self: "RelTag") -> str:
        """Объявление тега в начале файла."""
        return (
            f'AddRelTag("{self.repr_tag()}"'
            f"{self._repr_if_not_none(self.__text_color, 'textColor')}"
            f"{self._repr_if_not_none(self.__line_color, 'lineColor')}"
            f"{self._repr_if_not_none(self.__line_style, 'lineStyle')}"
            ")"
        )



"""Уровень 3 - conmponent."""

from .base import BaseC4Element as _BaseC4Element
from .base import BaseSprite as _BaseSprite
from .tag import ElementTag as _ElementTag


class BaseComponent(_BaseC4Element):
    """BaseComponent."""

    def __init__(
        self: "BaseComponent",
        label: str,
        techn: str | None,
        descr: str | None,
        sprite: _BaseSprite | None,
        tags: tuple[_ElementTag, ...] | None,
        link: str | None,
    ) -> None:
        """Создать BaseComponent."""
        super().__init__(
            label=label,
            sprite=sprite,
            link=link,
            tags=tags,
        )
        self.__techn = techn
        self.__descr = descr

    @property
    def _repr_inside_pths(self: "BaseComponent") -> str:
        """Возвращает содержимое внутри скобок."""
        return "({alias}{label}{techn}{descr}{sprite}{tags}{link})\n".format(
            alias=self.format_alias,
            label=self._format_label,
            techn=self._repr_if_not_none("techn", self.__techn),
            descr=self._repr_if_not_none("descr", self.__descr),
            sprite=self._format_sprite,
            tags=self._format_tags,
            link=self._format_link,
        )

    def __repr__(self: "BaseComponent") -> str:
        """Return string representation."""
        raise NotImplementedError("Метод не переопределен")


class Component(BaseComponent):
    """Component."""

    def __init__(
        self: "Component",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать Component."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "Component") -> str:
        """Return string representation."""
        return f"Component{self._repr_inside_pths}"


class ComponentDb(BaseComponent):
    """ComponentDb."""

    def __init__(
        self: "ComponentDb",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать ComponentDb."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "ComponentDb") -> str:
        """Return string representation."""
        return f"ComponentDb{self._repr_inside_pths}"


class ComponentQueue(BaseComponent):
    """ComponentQueue."""

    def __init__(
        self: "ComponentQueue",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать ComponentQueue."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "ComponentQueue") -> str:
        """Return string representation."""
        return f"ComponentQueue{self._repr_inside_pths}"


class ComponentExt(BaseComponent):
    """ComponentExt."""

    def __init__(
        self: "ComponentExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать ComponentExt."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "ComponentExt") -> str:
        """Return string representation."""
        return f"Component_Ext{self._repr_inside_pths}"


class ComponentDbExt(BaseComponent):
    """ComponentDbExt."""

    def __init__(
        self: "ComponentDbExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать ComponentDbExt."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "ComponentDbExt") -> str:
        """Return string representation."""
        return f"ComponentDb_Ext{self._repr_inside_pths}"


class ComponentQueueExt(BaseComponent):
    """ComponentDbExt."""

    def __init__(
        self: "ComponentQueueExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
    ) -> None:
        """Создать ComponentQueueExt."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
        )

    def __repr__(self: "ComponentQueueExt") -> str:
        """Return string representation."""
        return f"ComponentQueue_Ext{self._repr_inside_pths}"



"""Точка входа для генерации схем."""

import logging
import os
import os.path
import shutil
from importlib import import_module
from types import ModuleType

from konstantin_docs.dia.base import BaseDiagram


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())


class ModuleInfo:
    """Модуль для импорта."""

    __imported_module: ModuleType | None = None

    def __init__(
        self: "ModuleInfo",
        path_module: str,
        filename: str,
        path_src: str,
        path_dist: str,
    ) -> None:
        """Create ModuleInfo."""
        self.__path_module = path_module
        self.__filename = filename
        self.__path_src = path_src
        self.__path_dist = path_dist

    @property
    def imported(self: "ModuleInfo") -> ModuleType | None:
        """Возвращает импортированный модуль."""
        return self.__imported_module

    @property
    def path_module(self: "ModuleInfo") -> str:
        """Возвращает путь до исходной папки с модулем."""
        return self.__path_module

    @property
    def path_image(self: "ModuleInfo") -> str:
        """Возвращает путь до целевой папки с изображениями."""
        return self.__path_module.replace(self.__path_src, self.__path_dist)

    def try_import(self: "ModuleInfo") -> None:
        """Попытка импортировать модуль."""
        prefix = self.__path_to_import_prefix()
        if prefix == "":
            module_name = self.__filename
        else:
            module_name = prefix + "." + self.__filename
        logger.info("Попытка импорта %s", module_name)
        self.__imported_module = import_module(module_name)

    def __path_to_import_prefix(self: "ModuleInfo") -> str:
        """Путь до модуля в префикс для импорта модуля."""
        return self.__path_module.replace("/", ".")


def __scan_folder_for_modules(
    path_src: str,
    path_dist: str,
) -> list[ModuleInfo]:
    """Сканируем файлы в папке и создаем список потенциальных модулей."""
    logger.debug("Начинаем сканировать папку и искать модули")
    modules: list[ModuleInfo] = []
    for dirpath, _, filenames in os.walk(path_src):
        logger.debug("Сканирование папки, dirpath: %s", dirpath)
        for module in filenames:
            if module[-3:] != ".py":
                continue
            if module == "main.py":
                continue
            logger.debug("Найден модуль %s", module)
            modules.append(
                ModuleInfo(
                    path_module=dirpath,
                    filename=module[:-3],
                    path_src=path_src,
                    path_dist=path_dist,
                ),
            )
    return modules


def __dia_from_module(module: ModuleType) -> tuple[BaseDiagram]:
    """Список диаграмм из модуля."""
    dias: list[BaseDiagram] = []
    for item in module.__dict__.values():
        if isinstance(item, BaseDiagram):
            dias.append(item)
    return tuple(dias)


def __ignore_files(directory: str, files: list[str]) -> list[str]:
    """Игнорировать файлы при копировании дерева папок."""
    return [f for f in files if os.path.isfile(os.path.join(directory, f))]


def generate_images(path_src: str, path_dist: str) -> None:
    """Основная функция для генерации изображений.

    :param path_src: путь к папке с текстовым описанием диаграмм
    :param path_dist: путь к папке, куда будут сохраняться изображения
    """
    # удаляем целевую папку
    shutil.rmtree(path=path_dist, ignore_errors=True)
    # копируем структуру папок
    shutil.copytree(
        src=path_src,
        dst=path_dist,
        ignore=__ignore_files,
    )
    with open(
        f"{path_dist}/__this_folder_is_automatically_generated__",
        "w",
    ) as f:
        f.write("")
    # сканируем файлы в исходной папке и находим потенциальные модули
    imports = __scan_folder_for_modules(path_src, path_dist)
    # попытка импортировать модули
    imported: list[ModuleInfo] = []
    for mod in imports:
        try:
            mod.try_import()
            logger.debug("Успешно импортирован модуль: %s", mod.imported)
            imported.append(mod)
        except ImportError:
            continue
    # генерируем диаграммы
    for mod, _path in [(mod.imported, mod.path_image) for mod in imported]:
        if mod is None:
            continue
        dias_in_module = __dia_from_module(mod)
        logger.debug(
            "В модуле %s найдено диаграмм: %s",
            mod,
            len(dias_in_module),
        )
        for dia in dias_in_module:
            for dia_format in dia.get_images():
                with open(_path + "/" + dia_format.filename, "wb") as stream:
                    stream.write(dia_format.content)



"""Уровень 2 - container."""

from .base import BaseC4Element as _BaseC4Element
from .base import BaseSprite as _BaseSprite
from .base import BaseTag as _BaseTag
from .component import BaseComponent as _BaseComponent
from .tag import ElementTag as _ElementTag


class BaseContainer(_BaseC4Element):
    """BaseContainer."""

    def __init__(
        self: "BaseContainer",
        label: str,
        techn: str | None,
        descr: str | None,
        sprite: _BaseSprite | None,
        tags: tuple[_ElementTag, ...] | None,
        link: str | None,
        links_component: list[_BaseComponent] | None,
    ) -> None:
        """Создать BaseContainer."""
        super().__init__(
            label=label,
            sprite=sprite,
            link=link,
            tags=tags,
        )
        self.__techn = techn
        self.__descr = descr
        self.__links_component = links_component

    @property
    def _format_links_component(self: "BaseContainer") -> str:
        """Возвращает список вложенных компонентов в виде строки."""
        if self.__links_component is None:
            return ""
        links_component_str = "".join(
            [repr(component) for component in self.__links_component],
        )
        return f"{{\n\t{links_component_str}\n}}"

    @property
    def _repr_inside_pths(self: "BaseContainer") -> str:
        """Возвращает содержимое внутри скобок + вложенные компоненты."""
        return (
            "({alias}{label}{techn}{descr}{sprite}{tags}{link})"
            "{links_component}\n"
        ).format(
            alias=self.format_alias,
            label=self._format_label,
            techn=self._repr_if_not_none("techn", self.__techn),
            descr=self._repr_if_not_none("descr", self.__descr),
            sprite=self._format_sprite,
            tags=self._format_tags,
            link=self._format_link,
            links_component=self._format_links_component,
        )

    @_BaseC4Element.all_sprites.getter
    def all_sprites(self: "BaseContainer") -> list[_BaseSprite]:
        """Возвращает все спрайты."""
        sprites: list[_BaseSprite] = []
        sprites = super().all_sprites
        sprites.extend(
            [
                sprites
                for link in self.__links_component or []
                for sprites in link.all_sprites
            ],
        )
        return sprites

    @_BaseC4Element.all_tags.getter
    def all_tags(self: "BaseContainer") -> list[_BaseTag]:
        """Возвращает все теги."""
        tags: list[_BaseTag] = []
        tags = super().all_tags
        tags.extend(
            [
                tags
                for link in self.__links_component or []
                for tags in link.all_tags
            ],
        )
        return tags

    def __repr__(self: "BaseContainer") -> str:
        """Return string representation."""
        raise NotImplementedError("Метод не переопределен")


class Container(BaseContainer):
    """Container."""

    def __init__(
        self: "Container",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag, ...] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать Container."""
        super().__init__(
            label=label,
            techn=techn,
            descr=descr,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "Container") -> str:
        """Return string representation."""
        return f"Container{self._repr_inside_pths}"


class ContainerDb(BaseContainer):
    """ContainerDb."""

    def __init__(
        self: "ContainerDb",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerDb."""
        super().__init__(
            label=label,
            descr=descr,
            techn=techn,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerDb") -> str:
        """Return string representation."""
        return f"ContainerDb{self._repr_inside_pths}"


class ContainerQueue(BaseContainer):
    """ContainerQueue."""

    def __init__(
        self: "ContainerQueue",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerQueue."""
        super().__init__(
            label=label,
            descr=descr,
            techn=techn,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerQueue") -> str:
        """Return string representation."""
        return f"ContainerQueue{self._repr_inside_pths}"


class ContainerExt(BaseContainer):
    """ContainerExt."""

    def __init__(
        self: "ContainerExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerExt."""
        super().__init__(
            label=label,
            descr=descr,
            techn=techn,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerExt") -> str:
        """Return string representation."""
        return f"Container_Ext{self._repr_inside_pths}"


class ContainerDbExt(BaseContainer):
    """ContainerDbExt."""

    def __init__(
        self: "ContainerDbExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerDbExt."""
        super().__init__(
            label=label,
            descr=descr,
            techn=techn,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerDbExt") -> str:
        """Return string representation."""
        return f"ContainerDb_Ext{self._repr_inside_pths}"


class ContainerQueueExt(BaseContainer):
    """ContainerQueueExt."""

    def __init__(
        self: "ContainerQueueExt",
        label: str,
        techn: str = "",
        descr: str = "",
        sprite: _BaseSprite | None = None,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerQueueExt."""
        super().__init__(
            label=label,
            descr=descr,
            techn=techn,
            sprite=sprite,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerQueueExt") -> str:
        """Return string representation."""
        return f"ContainerQueue_Ext{self._repr_inside_pths}"


class ContainerBoundary(BaseContainer):
    """ContainerBoundary."""

    def __init__(
        self: "ContainerBoundary",
        label: str,
        tags: tuple[_ElementTag] | None = None,
        link: str | None = None,
        links_component: list[_BaseComponent] | None = None,
    ) -> None:
        """Создать ContainerBoundary."""
        super().__init__(
            label=label,
            descr=None,
            techn=None,
            sprite=None,
            link=link,
            tags=tags,
            links_component=links_component,
        )

    def __repr__(self: "ContainerBoundary") -> str:
        """Return string representation."""
        return f"Container_Boundary{self._repr_inside_pths}"



"""Interaction with https://kroki.io."""

import base64
import json
import zlib
from enum import Enum

import requests

URL = "https://kroki.io"


class DiagramTypes(Enum):
    """Diagram types."""

    GRAPHIZ = "graphiz"
    NWDIAG = "nwdiag"
    C4PLANTUML = "c4plantuml"
    MERMAID = "mermaid"


class OutputFormats(Enum):
    """Output formats."""

    PNG = "png"
    SVG = "svg"


def get_image(
    source: str,
    diagram_type: DiagramTypes,
    output_format: OutputFormats,
) -> bytes:
    """Возвращает изображение.

    :raises RuntimeError: отрицательный ответ от сервера
    :param source: текст диаграммы
    :param diagram_type: тип диаграммы
    :param output_format: формат изображения для генерации
    :return: изображение
    """
    req_type = "post"
    match req_type:
        case "post":
            response = requests.post(
                url=URL,
                data=json.dumps(
                    {
                        "diagram_source": source,
                        "diagram_type": diagram_type.value,
                        "output_format": output_format.value,
                    },
                ),
            )
        case "get":
            compressed = base64.urlsafe_b64encode(
                zlib.compress(source.encode("utf-8"), 9),
            ).decode("ascii")
            response = requests.get(
                url=URL
                + "/"
                + diagram_type.value
                + "/"
                + output_format.value
                + "/"
                + compressed,
            )
    if response.status_code != 200:
        raise RuntimeError(response.content)
    return response.content



"""Базовая диаграмма."""

from typing import NamedTuple

from typing_extensions import Self


class Image(NamedTuple):
    """Изображение с имененем файла."""

    filename: str
    content: bytes


class BaseDiagram:
    """Базовая диаграмма.

    Все диаграммы должны наследовать от этого класса
    """

    def __init__(
        self: "BaseDiagram",
        filename: str,
    ) -> None:
        """Создать объект базовой диаграммы.

        :param filename: имя создаваемого файла
        """
        self.__filename = filename

    @property
    def filename(self: "BaseDiagram") -> str:
        """Возвращает имя файла (без расширения).

        :return: имя файла
        """
        return self.__filename

    def get_images(self: Self) -> tuple[Image]:
        """Возвращает кортеж изображений.

        :raises NotImplementedError: Функция не определена
        """
        raise NotImplementedError("Функция не определена.")

    def _get_text_file(self: "BaseDiagram", ext: str = ".puml") -> Image:
        """Возвращает текстовый файл.

        :param ext: расширение для текстового файла
        :return: текстовый файл
        """
        return Image(
            filename=self.filename + ext,
            content=bytes(repr(self).encode()),
        )



"""Модель C4.

Описание - https://c4model.com/
Реализация на PlantUML - https://github.com/plantuml-stdlib/C4-PlantUML
"""

import logging

from konstantin_docs.dia.base import BaseDiagram as _BaseDiagram
from konstantin_docs.dia.base import Image
from konstantin_docs.service import kroki as _kroki

from . import component, container, context, rel, sprite, tag
from .base import BaseRelation as _BaseRelation
from .base import BaseSprite as _BaseSprite
from .base import BaseTag as _BaseTag

logger: logging.Logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
logger.addHandler(logging.StreamHandler())


# Diagram ---------------------------------------------------------------------

TEMPLATE_DIAGRAM: str = """@startuml

!include C4_Dynamic.puml
{sprites}
{title}
{tag}
{context}
{container}
{rels}
SHOW_LEGEND()
@enduml
"""


class C4(_BaseDiagram):
    """Диаграмма C4."""

    __title: str
    __links_context: list[context.BaseContext] | None
    __links_container: list[container.BaseContainer] | None
    __links_component: list[component.BaseComponent] | None
    __link_rels: list[_BaseRelation] | None

    def __init__(
        self: "C4",
        filename: str,
        title: str = "Diagram title",
        links_context: list[context.BaseContext] | None = None,
        links_container: list[container.BaseContainer] | None = None,
        links_component: list[component.BaseComponent] | None = None,
        links_rel: list[_BaseRelation] | None = None,
    ) -> None:
        """Создает объект диаграммы.

        :param filename: имя файла, без расширения
        :param title: заголовок диаграммы
        """
        super().__init__(filename)
        self.__title = title
        self.__links_context = links_context
        self.__links_container = links_container
        self.__links_component = links_component
        self.__link_rels = links_rel

    @property
    def _format_sprites(self: "C4") -> str:
        """Возвращает все спрайты.

        :return: форматированные спрайты
        """
        all_sprites: list[_BaseSprite] = []
        all_sprites.extend(
            [
                all_sprites
                for link in self.__links_context or []
                for all_sprites in link.all_sprites
            ],
        )
        all_sprites.extend(
            [
                all_sprites
                for link in self.__links_container or []
                for all_sprites in link.all_sprites
            ],
        )
        all_sprites.extend(
            [
                all_sprites
                for link in self.__links_component or []
                for all_sprites in link.all_sprites
            ],
        )
        common: set[str] = set()
        sprites: set[str] = set()
        for spr in all_sprites:
            common.add(spr.common)
            sprites.add(spr.sprite_full)
        out: str = ""
        if len(common) > 0:
            out += "\n".join(common) + "\n" + "\n".join(sprites)
        return out

    @property
    def _format_tags(self: "C4") -> str:
        """Возвращает объявление для всех тегов.

        :return:  форматированные теги
        """
        all_tags: list[_BaseTag] = []
        all_tags.extend(
            [
                all_tags
                for link in self.__links_context or []
                for all_tags in link.all_tags
            ],
        )
        all_tags.extend(
            [
                all_tags
                for link in self.__links_container or []
                for all_tags in link.all_tags
            ],
        )
        all_tags.extend(
            [
                all_tags
                for link in self.__links_component or []
                for all_tags in link.all_tags
            ],
        )
        repr_tags: set[str] = set([t.repr_declaration() for t in all_tags])
        out: str = ""
        if len(repr_tags) > 0:
            out += "\n".join(repr_tags)
        return out

    def get_images(self: "C4") -> tuple[Image]:
        """Возвращает кортеж файлов.

        :return: кортеж файлов
        """
        images: list[Image] = []
        text: str = repr(self)
        images.append(self._get_text_file(".puml"))
        try:
            for fmt in (_kroki.OutputFormats.PNG, _kroki.OutputFormats.SVG):
                images.append(
                    Image(
                        filename=self.filename + "." + fmt.value,
                        content=_kroki.get_image(
                            source=text,
                            diagram_type=_kroki.DiagramTypes.C4PLANTUML,
                            output_format=fmt,
                        ),
                    ),
                )
        except RuntimeError as exc:
            logger.exception(exc)
        return tuple(images)

    def __repr__(self: "C4") -> str:
        """Return string representation.

        :return: string representation
        """
        return TEMPLATE_DIAGRAM.format(
            tag=self._format_tags,
            sprites=self._format_sprites,
            title=f"title {self.__title}" if self.__title != "" else "",
            context="".join(
                [repr(context) for context in (self.__links_context or [])],
            ),
            container="".join(
                [
                    repr(container)
                    for container in (self.__links_container or [])
                ],
            ),
            rels="".join([repr(r) for r in (self.__link_rels or [])]),
        )


if __name__ == "__main__":
    _ = sprite.tupadr3.FontAwesome5(sprite.tupadr3.FontAwesome5Lib.AD)
    _ = tag.ElementTag(
        tag_stereo="123",
    )
    _ = rel.Rel(label="", links=(context.System("1"), context.System("2")))



"""devicons2 lib."""

from konstantin_docs.dia.c4.base import BaseSprites


class Devicons2Lib(BaseSprites):
    """Библиотека спрайтов."""

    AARCH64 = "aarch64"
    AFTEREFFECTS = "aftereffects"
    AMAZONWEBSERVICES_ORIGINAL = "amazonwebservices_original"
    AMAZONWEBSERVICES_WORDMARK = "amazonwebservices_wordmark"
    ANDROID = "android"
    ANDROID_WORDMARK = "android_wordmark"
    ANGULARJS = "angularjs"
    ANGULARJS_WORDMARK = "angularjs_wordmark"
    APACHE = "apache"
    APACHE_LINE = "apache_line"
    APACHE_LINE_WORDMARK = "apache_line_wordmark"
    APACHE_WORDMARK = "apache_wordmark"
    APPCELERATOR_ORIGINAL = "appcelerator_original"
    APPCELERATOR_WORDMARK = "appcelerator_wordmark"
    APPLE_ORIGINAL = "apple_original"
    APPWRITE = "appwrite"
    APPWRITE_WORDMARK = "appwrite_wordmark"
    ATOM_ORIGINAL = "atom_original"
    ATOM_ORIGINAL_WORDMARK = "atom_original_wordmark"
    BABEL = "babel"
    BACKBONEJS = "backbonejs"
    BACKBONEJS_WORDMARK = "backbonejs_wordmark"
    BASH = "bash"
    BEHANCE = "behance"
    BEHANCE_WORDMARK = "behance_wordmark"
    BITBUCKET_ORIGINAL = "bitbucket_original"
    BITBUCKET_ORIGINAL_WORDMARK = "bitbucket_original_wordmark"
    BOOTSTRAP = "bootstrap"
    BOOTSTRAP_WORDMARK = "bootstrap_wordmark"
    BOWER = "bower"
    BOWER_LINE = "bower_line"
    BOWER_LINE_WORDMARK = "bower_line_wordmark"
    BOWER_WORDMARK = "bower_wordmark"
    BULMA = "bulma"
    C = "c"
    C_LINE = "c_line"
    CAKEPHP = "cakephp"
    CAKEPHP_WORDMARK = "cakephp_wordmark"
    CEYLON = "ceylon"
    CHROME = "chrome"
    CHROME_WORDMARK = "chrome_wordmark"
    CIRCLECI = "circleci"
    CIRCLECI_WORDMARK = "circleci_wordmark"
    CLOJURE_LINE = "clojure_line"
    CLOJURESCRIPT = "clojurescript"
    CODECOV = "codecov"
    CODEIGNITER = "codeigniter"
    CODEIGNITER_WORDMARK = "codeigniter_wordmark"
    CODEPEN = "codepen"
    CODEPEN_ORIGINAL_WORDMARK = "codepen_original_wordmark"
    COFFEESCRIPT_ORIGINAL = "coffeescript_original"
    COFFEESCRIPT_ORIGINAL_WORDMARK = "coffeescript_original_wordmark"
    COMPOSER_LINE = "composer_line"
    COMPOSER_LINE_WORDMARK = "composer_line_wordmark"
    CONFLUENCE_ORIGINAL = "confluence_original"
    CONFLUENCE_ORIGINAL_WORDMARK = "confluence_original_wordmark"
    COUCHDB = "couchdb"
    COUCHDB_WORDMARK = "couchdb_wordmark"
    CPLUSPLUS = "cplusplus"
    CPLUSPLUS_LINE = "cplusplus_line"
    CRYSTAL_ORIGINAL = "crystal_original"
    CRYSTAL_ORIGINAL_WORDMARK = "crystal_original_wordmark"
    CSHARP = "csharp"
    CSHARP_LINE = "csharp_line"
    CSS3 = "css3"
    CSS3_WORDMARK = "css3_wordmark"
    CUCUMBER = "cucumber"
    CUCUMBER_WORDMARK = "cucumber_wordmark"
    D3JS = "d3js"
    D3JS_ORIGINAL = "d3js_original"
    DART = "dart"
    DART_WORDMARK = "dart_wordmark"
    DEBIAN = "debian"
    DEBIAN_WORDMARK = "debian_wordmark"
    DEVICON = "devicon"
    DEVICON_WORDMARK = "devicon_wordmark"
    DIGITALOCEAN = "digitalocean"
    DIGITALOCEAN_WORDMARK = "digitalocean_wordmark"
    DJANGO = "django"
    DJANGO_LINE = "django_line"
    DOCKER = "docker"
    DOCKER_WORDMARK = "docker_wordmark"
    DOCTRINE = "doctrine"
    DOCTRINE_LINE = "doctrine_line"
    DOCTRINE_LINE_WORDMARK = "doctrine_line_wordmark"
    DOCTRINE_WORDMARK = "doctrine_wordmark"
    DOT_NET = "dot_net"
    DOT_NET_WORDMARK = "dot_net_wordmark"
    DOTNETCORE = "dotnetcore"
    DRUPAL = "drupal"
    DRUPAL_WORDMARK = "drupal_wordmark"
    ELECTRON_ORIGINAL = "electron_original"
    ELECTRON_ORIGINAL_WORDMARK = "electron_original_wordmark"
    ELEVENTY = "eleventy"
    ELIXIR = "elixir"
    ELIXIR_WORDMARK = "elixir_wordmark"
    ELM = "elm"
    ELM_WORDMARK = "elm_wordmark"
    EMBEDDEDC = "embeddedc"
    EMBEDDEDC_WORDMARK = "embeddedc_wordmark"
    EMBER_ORIGINAL_WORDMARK = "ember_original_wordmark"
    ERLANG = "erlang"
    ERLANG_WORDMARK = "erlang_wordmark"
    EXPRESS_ORIGINAL = "express_original"
    EXPRESS_ORIGINAL_WORDMARK = "express_original_wordmark"
    FACEBOOK = "facebook"
    FIGMA = "figma"
    FIREBASE = "firebase"
    FIREBASE_WORDMARK = "firebase_wordmark"
    FIREFOX = "firefox"
    FIREFOX_WORDMARK = "firefox_wordmark"
    FLASK_ORIGINAL = "flask_original"
    FLASK_ORIGINAL_WORDMARK = "flask_original_wordmark"
    FLUTTER = "flutter"
    FOUNDATION = "foundation"
    FOUNDATION_WORDMARK = "foundation_wordmark"
    FSHARP = "fsharp"
    GATLING = "gatling"
    GATLING_WORDMARK = "gatling_wordmark"
    GATSBY = "gatsby"
    GATSBY_WORDMARK = "gatsby_wordmark"
    GIMP = "gimp"
    GIT = "git"
    GIT_WORDMARK = "git_wordmark"
    GITHUB_ORIGINAL = "github_original"
    GITHUB_ORIGINAL_WORDMARK = "github_original_wordmark"
    GITLAB = "gitlab"
    GITLAB_WORDMARK = "gitlab_wordmark"
    GITTER = "gitter"
    GITTER_WORDMARK = "gitter_wordmark"
    GO = "go"
    GO_LINE = "go_line"
    GODOT = "godot"
    GODOT_WORDMARK = "godot_wordmark"
    GOOGLE = "google"
    GOOGLE_WORDMARK = "google_wordmark"
    GOOGLECLOUD = "googlecloud"
    GOOGLECLOUD_WORDMARK = "googlecloud_wordmark"
    GRADLE = "gradle"
    GRADLE_WORDMARK = "gradle_wordmark"
    GRAILS = "grails"
    GRAPHQL = "graphql"
    GRAPHQL_WORDMARK = "graphql_wordmark"
    GROOVY = "groovy"
    GRUNT = "grunt"
    GRUNT_LINE = "grunt_line"
    GRUNT_LINE_WORDMARK = "grunt_line_wordmark"
    GRUNT_WORDMARK = "grunt_wordmark"
    GULP = "gulp"
    HANDLEBARS = "handlebars"
    HANDLEBARS_WORDMARK = "handlebars_wordmark"
    HASKELL = "haskell"
    HASKELL_WORDMARK = "haskell_wordmark"
    HAXE = "haxe"
    HEROKU = "heroku"
    HEROKU_LINE = "heroku_line"
    HEROKU_LINE_WORDMARK = "heroku_line_wordmark"
    HEROKU_WORDMARK = "heroku_wordmark"
    HTML5 = "html5"
    HTML5_WORDMARK = "html5_wordmark"
    IE10_ORIGINAL = "ie10_original"
    ILLUSTRATOR = "illustrator"
    ILLUSTRATOR_LINE = "illustrator_line"
    INDEX = "index"
    INKSCAPE = "inkscape"
    INKSCAPE_WORDMARK = "inkscape_wordmark"
    INTELLIJ = "intellij"
    INTELLIJ_WORDMARK = "intellij_wordmark"
    IONIC_ORIGINAL = "ionic_original"
    IONIC_ORIGINAL_WORDMARK = "ionic_original_wordmark"
    JASMINE = "jasmine"
    JASMINE_WORDMARK = "jasmine_wordmark"
    JAVA = "java"
    JAVA_WORDMARK = "java_wordmark"
    JAVASCRIPT = "javascript"
    JEET = "jeet"
    JEET_WORDMARK = "jeet_wordmark"
    JENKINS = "jenkins"
    JENKINS_LINE = "jenkins_line"
    JEST = "jest"
    JETBRAINS = "jetbrains"
    JQUERY = "jquery"
    JQUERY_WORDMARK = "jquery_wordmark"
    JULIA = "julia"
    JULIA_WORDMARK = "julia_wordmark"
    JUPYTER = "jupyter"
    JUPYTER_WORDMARK = "jupyter_wordmark"
    KARMA = "karma"
    KNOCKOUT_WORDMARK = "knockout_wordmark"
    KOTLIN = "kotlin"
    KOTLIN_WORDMARK = "kotlin_wordmark"
    KRAKENJS = "krakenjs"
    KRAKENJS_WORDMARK = "krakenjs_wordmark"
    KUBERNETES = "kubernetes"
    KUBERNETES_WORDMARK = "kubernetes_wordmark"
    LABVIEW = "labview"
    LABVIEW_WORDMARK = "labview_wordmark"
    LARAVEL = "laravel"
    LARAVEL_WORDMARK = "laravel_wordmark"
    LESS_WORDMARK = "less_wordmark"
    LINKEDIN = "linkedin"
    LINKEDIN_WORDMARK = "linkedin_wordmark"
    LINUX = "linux"
    LUA = "lua"
    LUA_WORDMARK = "lua_wordmark"
    MAGENTO_LINE = "magento_line"
    MAGENTO_ORIGINAL = "magento_original"
    MAGENTO_ORIGINAL_WORDMARK = "magento_original_wordmark"
    MATERIALUI = "materialui"
    MATLAB = "matlab"
    MATLAB_LINE = "matlab_line"
    METEOR = "meteor"
    METEOR_WORDMARK = "meteor_wordmark"
    MICROSOFTSQLSERVER = "microsoftsqlserver"
    MICROSOFTSQLSERVER_WORDMARK = "microsoftsqlserver_wordmark"
    MINITAB = "minitab"
    MOCHA = "mocha"
    MODX = "modx"
    MODX_WORDMARK = "modx_wordmark"
    MONGODB = "mongodb"
    MONGODB_WORDMARK = "mongodb_wordmark"
    MOODLE = "moodle"
    MOODLE_WORDMARK = "moodle_wordmark"
    MYSQL = "mysql"
    MYSQL_WORDMARK = "mysql_wordmark"
    NESTJS = "nestjs"
    NESTJS_WORDMARK = "nestjs_wordmark"
    NEXTJS_LINE = "nextjs_line"
    NEXTJS_ORIGINAL = "nextjs_original"
    NEXTJS_ORIGINAL_WORDMARK = "nextjs_original_wordmark"
    NGINX_ORIGINAL = "nginx_original"
    NIXOS = "nixos"
    NIXOS_WORDMARK = "nixos_wordmark"
    NODEJS = "nodejs"
    NODEJS_WORDMARK = "nodejs_wordmark"
    NODEWEBKIT = "nodewebkit"
    NODEWEBKIT_LINE = "nodewebkit_line"
    NODEWEBKIT_LINE_WORDMARK = "nodewebkit_line_wordmark"
    NODEWEBKIT_WORDMARK = "nodewebkit_wordmark"
    NPM_ORIGINAL_WORDMARK = "npm_original_wordmark"
    OBJECTIVEC = "objectivec"
    OCAML = "ocaml"
    OCAML_WORDMARK = "ocaml_wordmark"
    ORACLE_ORIGINAL = "oracle_original"
    PERL = "perl"
    PHALCON = "phalcon"
    PHOENIX = "phoenix"
    PHOENIX_WORDMARK = "phoenix_wordmark"
    PHOTOSHOP = "photoshop"
    PHOTOSHOP_LINE = "photoshop_line"
    PHP = "php"
    PHPSTORM = "phpstorm"
    PHPSTORM_WORDMARK = "phpstorm_wordmark"
    POSTGRESQL = "postgresql"
    POSTGRESQL_WORDMARK = "postgresql_wordmark"
    PREMIEREPRO = "premierepro"
    PROTRACTOR = "protractor"
    PROTRACTOR_WORDMARK = "protractor_wordmark"
    PYCHARM = "pycharm"
    PYCHARM_WORDMARK = "pycharm_wordmark"
    PYTHON = "python"
    PYTHON_WORDMARK = "python_wordmark"
    R = "r"
    R_ORIGINAL = "r_original"
    RAILS = "rails"
    RAILS_WORDMARK = "rails_wordmark"
    RASPBERRYPI_LINE = "raspberrypi_line"
    RASPBERRYPI_LINE_WORDMARK = "raspberrypi_line_wordmark"
    REACT_ORIGINAL = "react_original"
    REACT_ORIGINAL_WORDMARK = "react_original_wordmark"
    REDHAT = "redhat"
    REDHAT_WORDMARK = "redhat_wordmark"
    REDIS = "redis"
    REDIS_WORDMARK = "redis_wordmark"
    REDUX_ORIGINAL = "redux_original"
    ROCKSDB = "rocksdb"
    RSTUDIO = "rstudio"
    RUBY = "ruby"
    RUBY_WORDMARK = "ruby_wordmark"
    RUBYMINE = "rubymine"
    RUBYMINE_WORDMARK = "rubymine_wordmark"
    RUST = "rust"
    SAFARI = "safari"
    SAFARI_LINE = "safari_line"
    SAFARI_LINE_WORDMARK = "safari_line_wordmark"
    SAFARI_WORDMARK = "safari_wordmark"
    SALESFORCE = "salesforce"
    SASS_ORIGINAL = "sass_original"
    SCALA = "scala"
    SCALA_WORDMARK = "scala_wordmark"
    SEQUELIZE = "sequelize"
    SEQUELIZE_WORDMARK = "sequelize_wordmark"
    SHOPWARE_ORIGINAL = "shopware_original"
    SHOPWARE_ORIGINAL_WORDMARK = "shopware_original_wordmark"
    SKETCH_LINE = "sketch_line"
    SKETCH_LINE_WORDMARK = "sketch_line_wordmark"
    SLACK = "slack"
    SLACK_WORDMARK = "slack_wordmark"
    SOURCETREE_ORIGINAL = "sourcetree_original"
    SOURCETREE_ORIGINAL_WORDMARK = "sourcetree_original_wordmark"
    SPRING = "spring"
    SPRING_WORDMARK = "spring_wordmark"
    SPSS = "spss"
    SQLALCHEMY = "sqlalchemy"
    SQLALCHEMY_ORIGINAL_WORDMARK = "sqlalchemy_original_wordmark"
    SSH_ORIGINAL = "ssh_original"
    SSH_ORIGINAL_WORDMARK = "ssh_original_wordmark"
    STYLUS_ORIGINAL = "stylus_original"
    SWIFT = "swift"
    SWIFT_WORDMARK = "swift_wordmark"
    SYMFONY_ORIGINAL = "symfony_original"
    SYMFONY_ORIGINAL_WORDMARK = "symfony_original_wordmark"
    TAILWINDCSS = "tailwindcss"
    TAILWINDCSS_ORIGINAL_WORDMARK = "tailwindcss_original_wordmark"
    TENSORFLOW_LINE = "tensorflow_line"
    TENSORFLOW_LINE_WORDMARK = "tensorflow_line_wordmark"
    TENSORFLOW_ORIGINAL = "tensorflow_original"
    TENSORFLOW_ORIGINAL_WORDMARK = "tensorflow_original_wordmark"
    THEALGORITHMS = "thealgorithms"
    THEALGORITHMS_WORDMARK = "thealgorithms_wordmark"
    TOMCAT_LINE = "tomcat_line"
    TOMCAT_LINE_WORDMARK = "tomcat_line_wordmark"
    TRAVIS = "travis"
    TRAVIS_WORDMARK = "travis_wordmark"
    TRELLO = "trello"
    TRELLO_WORDMARK = "trello_wordmark"
    TWITTER_ORIGINAL = "twitter_original"
    TYPESCRIPT = "typescript"
    TYPO3 = "typo3"
    TYPO3_WORDMARK = "typo3_wordmark"
    UBUNTU = "ubuntu"
    UBUNTU_WORDMARK = "ubuntu_wordmark"
    UNIX_ORIGINAL = "unix_original"
    UWSGI = "uwsgi"
    VAGRANT = "vagrant"
    VAGRANT_WORDMARK = "vagrant_wordmark"
    VIM = "vim"
    VISUALSTUDIO = "visualstudio"
    VISUALSTUDIO_WORDMARK = "visualstudio_wordmark"
    VSCODE = "vscode"
    VSCODE_WORDMARK = "vscode_wordmark"
    VUEJS = "vuejs"
    VUEJS_LINE = "vuejs_line"
    VUEJS_LINE_WORDMARK = "vuejs_line_wordmark"
    VUEJS_WORDMARK = "vuejs_wordmark"
    VUESTOREFRONT = "vuestorefront"
    WEBLATE = "weblate"
    WEBLATE_WORDMARK = "weblate_wordmark"
    WEBPACK = "webpack"
    WEBPACK_WORDMARK = "webpack_wordmark"
    WEBSTORM = "webstorm"
    WEBSTORM_WORDMARK = "webstorm_wordmark"
    WINDOWS8_ORIGINAL = "windows8_original"
    WINDOWS8_ORIGINAL_WORDMARK = "windows8_original_wordmark"
    WOOCOMMERCE = "woocommerce"
    WOOCOMMERCE_WORDMARK = "woocommerce_wordmark"
    WORDPRESS = "wordpress"
    WORDPRESS_WORDMARK = "wordpress_wordmark"
    XD = "xd"
    XD_LINE = "xd_line"
    YARN = "yarn"
    YARN_WORDMARK = "yarn_wordmark"
    YII = "yii"
    YII_WORDMARK = "yii_wordmark"
    YUNOHOST = "yunohost"
    ZEND = "zend"
    ZEND_WORDMARK = "zend_wordmark"
    ZIG_ORIGINAL = "zig_original"
    ZIG_WORDMARK = "zig_wordmark"



"""devicons lib."""

from konstantin_docs.dia.c4.base import BaseSprites


class DeviconsLib(BaseSprites):
    """Библиотека спрайтов."""

    ANDROID = "android"
    ANGULAR = "angular"
    APPCELERATOR = "appcelerator"
    APPLE = "apple"
    APPSTORE = "appstore"
    APTANA = "aptana"
    ASTERISK = "asterisk"
    ATLASSIAN = "atlassian"
    ATOM = "atom"
    AWS = "aws"
    BACKBONE = "backbone"
    BING_SMALL = "bing_small"
    BINTRAY = "bintray"
    BITBUCKET = "bitbucket"
    BLACKBERRY = "blackberry"
    BOOTSTRAP = "bootstrap"
    BOWER = "bower"
    BRACKETS = "brackets"
    BUGSENSE = "bugsense"
    CELLULOID = "celluloid"
    CHROME = "chrome"
    CISCO = "cisco"
    CLOJURE = "clojure"
    CLOJURE_ALT = "clojure_alt"
    CLOUD9 = "cloud9"
    CODA = "coda"
    CODE = "code"
    CODE_BADGE = "code_badge"
    CODEIGNITER = "codeigniter"
    CODEPEN = "codepen"
    CODROPS = "codrops"
    COFFEESCRIPT = "coffeescript"
    COMPASS = "compass"
    COMPOSER = "composer"
    CREATIVECOMMONS = "creativecommons"
    CREATIVECOMMONS_BADGE = "creativecommons_badge"
    CSS3 = "css3"
    CSS3_FULL = "css3_full"
    CSS_TRICKS = "css_tricks"
    CSSDECK = "cssdeck"
    DART = "dart"
    DATABASE = "database"
    DEBIAN = "debian"
    DIGITAL_OCEAN = "digital_ocean"
    DJANGO = "django"
    DLANG = "dlang"
    DOCKER = "docker"
    DOCTRINE = "doctrine"
    DOJO = "dojo"
    DOTNET = "dotnet"
    DREAMWEAVER = "dreamweaver"
    DROPBOX = "dropbox"
    DRUPAL = "drupal"
    ECLIPSE = "eclipse"
    EMBER = "ember"
    ENVATO = "envato"
    ERLANG = "erlang"
    EXTJS = "extjs"
    FIREBASE = "firebase"
    FIREFOX = "firefox"
    FSHARP = "fsharp"
    GHOST = "ghost"
    GHOST_SMALL = "ghost_small"
    GIT = "git"
    GIT_BRANCH = "git_branch"
    GIT_COMMIT = "git_commit"
    GIT_COMPARE = "git_compare"
    GIT_MERGE = "git_merge"
    GIT_PULL_REQUEST = "git_pull_request"
    GITHUB = "github"
    GITHUB_ALT = "github_alt"
    GITHUB_BADGE = "github_badge"
    GITHUB_FULL = "github_full"
    GNU = "gnu"
    GO = "go"
    GOOGLE_ANALYTICS = "google_analytics"
    GOOGLE_CLOUD_PLATFORM = "google_cloud_platform"
    GOOGLE_DRIVE = "google_drive"
    GRAILS = "grails"
    GROOVY = "groovy"
    GRUNT = "grunt"
    GULP = "gulp"
    HACKERNEWS = "hackernews"
    HASKELL = "haskell"
    HEROKU = "heroku"
    HTML5 = "html5"
    HTML5_3D_EFFECTS = "html5_3d_effects"
    HTML5_CONNECTIVITY = "html5_connectivity"
    HTML5_DEVICE_ACCESS = "html5_device_access"
    HTML5_MULTIMEDIA = "html5_multimedia"
    IE = "ie"
    ILLUSTRATOR = "illustrator"
    INDEX = "index"
    INTELLIJ = "intellij"
    IONIC = "ionic"
    JAVA = "java"
    JAVASCRIPT = "javascript"
    JAVASCRIPT_BADGE = "javascript_badge"
    JAVASCRIPT_SHIELD = "javascript_shield"
    JEKYLL_SMALL = "jekyll_small"
    JENKINS = "jenkins"
    JIRA = "jira"
    JOOMLA = "joomla"
    JQUERY = "jquery"
    JQUERY_UI = "jquery_ui"
    KOMODO = "komodo"
    KRAKENJS = "krakenjs"
    KRAKENJS_BADGE = "krakenjs_badge"
    LARAVEL = "laravel"
    LESS = "less"
    LINUX = "linux"
    MAGENTO = "magento"
    MAILCHIMP = "mailchimp"
    MARKDOWN = "markdown"
    MATERIALIZECSS = "materializecss"
    METEOR = "meteor"
    METEORFULL = "meteorfull"
    MITLICENCE = "mitlicence"
    MODERNIZR = "modernizr"
    MONGODB = "mongodb"
    MOOTOOLS = "mootools"
    MOOTOOLS_BADGE = "mootools_badge"
    MOZILLA = "mozilla"
    MSQL_SERVER = "msql_server"
    MYSQL = "mysql"
    NANCY = "nancy"
    NETBEANS = "netbeans"
    NETMAGAZINE = "netmagazine"
    NGINX = "nginx"
    NODEJS = "nodejs"
    NODEJS_SMALL = "nodejs_small"
    NPM = "npm"
    ONEDRIVE = "onedrive"
    OPENSHIFT = "openshift"
    OPENSOURCE = "opensource"
    OPERA = "opera"
    PERL = "perl"
    PHONEGAP = "phonegap"
    PHOTOSHOP = "photoshop"
    PHP = "php"
    POSTGRESQL = "postgresql"
    PROLOG = "prolog"
    PYTHON = "python"
    RACKSPACE = "rackspace"
    RAPHAEL = "raphael"
    RASPBERRY_PI = "raspberry_pi"
    REACT = "react"
    REDHAT = "redhat"
    REDIS = "redis"
    REQUIREJS = "requirejs"
    RESPONSIVE = "responsive"
    RUBY = "ruby"
    RUBY_ON_RAILS = "ruby_on_rails"
    RUBY_ROUGH = "ruby_rough"
    RUST = "rust"
    SAFARI = "safari"
    SASS = "sass"
    SCALA = "scala"
    SCRIPTCS = "scriptcs"
    SCRUM = "scrum"
    SENCHATOUCH = "senchatouch"
    SIZZLEJS = "sizzlejs"
    SMASHING_MAGAZINE = "smashing_magazine"
    SNAP_SVG = "snap_svg"
    SQLLITE = "sqllite"
    STACKOVERFLOW = "stackoverflow"
    STREAMLINE = "streamline"
    STYLUS = "stylus"
    SUBLIME = "sublime"
    SWIFT = "swift"
    SYMFONY = "symfony"
    SYMFONY_BADGE = "symfony_badge"
    TECHCRUNCH = "techcrunch"
    TERMINAL = "terminal"
    TERMINAL_BADGE = "terminal_badge"
    TRAVIS = "travis"
    TRELLO = "trello"
    TYPO3 = "typo3"
    UBUNTU = "ubuntu"
    UIKIT = "uikit"
    UNITY_SMALL = "unity_small"
    VIM = "vim"
    VISUALSTUDIO = "visualstudio"
    W3C = "w3c"
    WEBPLATFORM = "webplatform"
    WINDOWS = "windows"
    WORDPRESS = "wordpress"
    YAHOO = "yahoo"
    YAHOO_SMALL = "yahoo_small"
    YEOMAN = "yeoman"
    YII = "yii"
    ZEND = "zend"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['konstantin_docs',
 'konstantin_docs.dia',
 'konstantin_docs.dia.c4',
 'konstantin_docs.dia.c4.sprite_lib',
 'konstantin_docs.dia.c4.sprite_lib.tupadr3_lib',
 'konstantin_docs.dia.mermaid_er',
 'konstantin_docs.dia.mermaid_state',
 'konstantin_docs.service']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'typing_extensions']

setup_kwargs = {
    'name': 'konstantin-docs',
    'version': '0.0.13',
    'description': 'Генерация документации',
    'long_description': '# kroki-python\n\nБиблиотека для генерации диаграмм из текстового описания.\n\nДиаграммы описываются объектами python. Далее геренируются изображения с помощью https://kroki.io.\n\n## Как использовать\n\n1. Создать две папки:\n\n   - dia_src - папка с исходным описанием\n   - dia_dist - папка со сгенерированными изображениями\n\n2. В папке dia_src создаются py-файлы. Названия файлов - произвольные. Можно создавать подкаталоги - структура каталогов будет скопирована в целевую папку dia_dist. Примеры создания можно посмотреть в тестовых диаграммах [пакета](https://github.com/Konstantin-Dudersky/konstantin_docs/tree/main/test).\n\n3. Для генерации можно создать задачу poetepoet. Прописать в файле pyproject.toml:\n\n   ```toml\n   [tool.poetry.dependencies]\n   konstantin_docs = "*"\n   poethepoet = "*"\n   \n   [tool.poe.tasks]\n   docs = {script = "konstantin_docs.main:generate_images(\'dia_src\', \'dia_dist\')"}\n   ```\n\n4. Запустить командой:\n\n    ```sh\n    poetry run poe docs\n    ```\n\n5. Дополнительно можно создать задачу в vscode. Для этого в файле .vscode/tasks.json:\n\n   ```json\n   {\n     "version": "2.0.0",\n     "tasks": [\n       {\n         "label": "docs",\n         "type": "shell",\n         "command": "poetry run poe docs"\n       }\n     ]\n   }\n   ```\n\n   Запускать командой F1 -> Task: Run task -> docs\n\n\n',
    'author': 'Konstantin-Dudersky',
    'author_email': 'Konstantin.Dudersky@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Konstantin-Dudersky/konstantin_docs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



r"""Sprite.

Для генерации перечислений на основе списка файлов:
- в папке с файлами создать файл _generate.py:
-------------------------------------------------------------------------------
import os

OUT_FILENAME = "_enum.txt"
SCRIPT_FILENAME = "_generate.py"

sprites: set[str] = set()
for _, _, filenames in os.walk(os.getcwd()):
    # filenames: list[str] = filenames
    for filename in filenames:
        if filename in [OUT_FILENAME, SCRIPT_FILENAME]:
            continue
        sprite_parts = filename.split(".")[:-1]
        sprites.add(".".join(sprite_parts))

with open(OUT_FILENAME, "w", encoding="utf-8") as stream:
    for sprite in sorted(sprites):
        line = f'    {sprite.upper()} = "{sprite}"\n'
        stream.write(line)

-------------------------------------------------------------------------------
- настроить переменные BASE и PATH
- запустить командой python3 _generate.py
- содержимое файла _enum.txt вставить в перечиление
"""
from .sprite_lib import tupadr3

if __name__ == "__main__":
    _ = tupadr3.FontAwesome5(tupadr3.FontAwesome5Lib.AD)
